
/* Keep track of which switch we have set:                 */

static int led_switches = 0xffffffff;               /* all off       */

/* Since led startup is unknown, reset them to the default */

void led_init(void)
{
   volatile unsigned int *digio = (unsigned int *) 0x02220000;

  *digio = led_switches;
}

/* Digital output pin i needs to be set to 0 to illuminate */

void led_on(int bit)
{
   volatile unsigned int *digio = (unsigned int *) 0x02220000;

   led_switches &= ~(1<<bit);
  *digio = led_switches;
}

/* Digital output pin i needs to be set to 1 to stop illuminating */

void led_off(int bit)
{
   volatile unsigned int *digio = (unsigned int *) 0x02220000;
   
   led_switches |=  (1<<bit);
  *digio = led_switches;
}

/*
   Direct host fifo access ---------------------------------------------------
*/

#define HC6EMPTY                        0x0001
#define HC6HALFFULL             0x0002
#define HC6ALMOSTFULL           0x0004
#define HC6FULL                 0x0008

#define C6HEMPTY                        0x0080
#define C6HHALFFULL             0x0040
#define C6HALMOSTFULL           0x0020
#define C6HFULL                 0x0010

#define EIGHTK                  0x2000
#define SEVENK                  0x1C00
#define SIXK                    0x1800
#define FIVEK                   0x1400
#define FOURK                   0x1000

/*
   host_link_in, Parallel C syntax ----------------------------------------------
*/

void host_link_in(int len, unsigned int *buf)
{
   volatile unsigned int *host_fifo = (unsigned int *) 0x02100000;
   volatile unsigned int *flags     = (unsigned int *) 0x02200000;
   volatile unsigned int r;
   int i=0, j;

   len = (len+3)/4;
   while (len > 0)
   {
      while (!(*flags & HC6EMPTY)) asm("   NOP 5");
      r = *flags;

      if (!(r & HC6FULL))
      {
	 /* 8K words may be read */
	 if (len > EIGHTK)
	 {
	    for (j=0; j<EIGHTK; j++) buf[i++] = *host_fifo;
	    len -= EIGHTK;
	 }
	 else
	 {
	    for (j=0; j<len   ; j++) buf[i++] = *host_fifo;
	    len  = 0;
	 }
      }
      else
      if (!(r & HC6ALMOSTFULL))
      {
	 /* 7K words may be read */
	 if (len > SEVENK)
	 {
	    for (j=0; j<SEVENK; j++) buf[i++] = *host_fifo;
	    len -= SEVENK;
	 }
	 else
	 {
	    for (j=0; j<len   ; j++) buf[i++] = *host_fifo;
	    len  = 0;
	 }
      }
      else
      if (!(r & HC6HALFFULL))
      {
	 /* 4K words may be read */
	 if (len > FOURK)
	 {
	    for (j=0; j<FOURK ; j++) buf[i++] = *host_fifo;
	    len -= FOURK;
	 }
	 else
	 {
	    for (j=0; j<len   ; j++) buf[i++] = *host_fifo;
	    len  = 0;
	 }
      }
      else
      {
	 /* 1 word may be read */
	 buf[i++] = *host_fifo;
	 len   -=  1;
      }
   }
}

unsigned int buf[9600];
 
void main(void)
{
	int i,j;

	led_init();                                  /* Sets the LEDs to a known state */

	for (j=0; j<4; j++)
	{
	   led_on(j);
	   for (i=0; i<100000;i++){}
	   led_off(j);
	   for (i=0; i<100000;i++){}
	}

	while (1)
	{
		host_link_in(9600*4, buf);
		led_on(j);
		host_link_in(9600*4, buf);
		led_off(j);
	}
}


