
/*

	WRITES.C 	Readspeed for HUNT ENGINEERING API

	Test the write (Host to 'C4x) speed of an implementation

	  -Jan-96	V1.0	DSB	Ported from the original to the API
	13-Jun-97	V1.1	DSB	Cosmetic tidy up prior to API 1.2 release
	08-Jul-97	V1.2	DSB	Ensured it keeps the reset Comport Open
	02-Oct-1997	V1.3	DSB	Ported to Borland Builder
*/

#define _VXWORKS

#ifdef _VXWORKS
#include "VxWorks.h"
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include "dosFsLib.h"
#include "ioLib.h"
#include "time.h"
#include "timers.h"
#include "heapi/heapi.h"
#include "header.h"

extern sysClkRateGet();

#else

#include <vcl\condefs.h>
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <dos.h>
#include <time.h>

#include "heapi.h"
#include "header.h"

#endif

#if MSDOS
#define READ_BIN "rb"
#else
#define READ_BIN "r"
#endif

/*--------------------------------------------------------------------------*/
/* LOADER DEFINITIONS                                                       */
/*--------------------------------------------------------------------------*/
FILE *fopen();
char *strrchr() /*, *malloc(), *realloc()*/;

unsigned long reloc = 0;                 /* RELOCATION AMOUNT               */
int           quiet = 0;                 /* NO BANNER                       */

HE_HANDLE 		hDevice, hReset;
HE_IOSTATUS	 	ReadIoStatus, WriteIoStatus;
HE_DWORD 		Status;
HE_BOOLEAN		UseComportA;
HE_MEMHANDLE		memHandle = NULL;
HE_PDWORDBUFFER		test;

void OurError(HE_DWORD errorcode)
{
	char text[120];
	HeErr2Text(Status, text);
	printf("Failed - %s, OsError = %ld\n", text, errorcode);
	if(!UseComportA) Status = HeClose(&hReset);
	Status = HeClose(&hDevice);
	exit(1);
}

int HeApiWrite(char *WriteBuffer, int size)
{
	HE_DWORD Status;

	Status = HeWrite(hDevice, WriteBuffer, size, WriteIoStatus);
	if (Status == HE_IoInProgress)
	{
		Status = HeWaitForIo(hDevice, WriteIoStatus);
	}
	if (Status != HE_OK)
	{
		OurError(HeGetLastOsError(hDevice));
		return 0;
	}
    return 1;
}

void error(HE_DWORD errorcode)
{
	char text[120];
	HeErr2Text(Status, text);
	printf("Failed - %s, OsError = %ld\n", text, errorcode);
	Status = HeClose(&hDevice);
	if(!UseComportA) Status = HeClose(&hReset);
	exit(1);
}

HE_DWORD MyWriteWord(HE_HANDLE hDevice, HE_DWORD *data)
{
	Status = HeWrite(hDevice, data, 4, WriteIoStatus);
	if(Status != HE_OK) {
		if(Status == HE_IoInProgress) {
			Status = HeWaitForIo(hDevice, WriteIoStatus);
		}
	}
	return Status;
}

HE_DWORD b0,b1,b2, BlockSize;

/*********************************************************************
 *
 *	Function : write_speed()
 *
 *********************************************************************/

void write_speed(void)
{
    HE_DWORD i, j;
    clock_t start,end, time_in;

    printf("Testing...\n");

    for (i=b0;i<=b2;i+=b1){
		start = clock();
		if(BlockSize==0) {
			for(j=0; j<i; j++) {
				Status = MyWriteWord(hDevice, &test[j]);
				if(Status != HE_OK) error(HeGetLastOsError(hDevice));
			}
		}
		else {	
			Status = HeWrite(hDevice, test, i<<2, WriteIoStatus);
			if(Status == HE_IoInProgress) {
				Status = HeWaitForIo(hDevice, WriteIoStatus);
			}
			if(Status != HE_OK) error(HeGetLastOsError(hDevice));
		}

		end = clock();
		printf("Writes Transfer size %ld DWORDS ",i);
		time_in = end - start;

		if(time_in == 0) time_in = 1;
		printf("in %d ticks, ",time_in);
		printf("   Speed: %.2f KBytes/sec\n",((float)(i*CLOCKS_PER_SEC)*4.0)/
				((float)time_in*1024.0));
    }

    return;
}

void tolwr(char *s)
{
   int i=0;
   while (s[i]!=0)
   {
      s[i]=tolower(s[i]);
      i++;
   }
}

/*********************************************************************/
/*	main							     */
/*********************************************************************/

HE_DWORD switches[4];

#ifdef _VXWORKS
void Writes(char *dev, int brd, char *comm, int strt, int inc, int end, char *mm, char *ints, char *irq)
{
	HE_DWORD	i, config;
	HE_WORD		Board;
	HE_BOOLEAN	ConfigSupported = FALSE;
	char		*outfile, *devname;
	char		Comport[20];
	int		nul = 0;
	int		heron_id = 1;
	struct
	{
		int		read_ints;
		int		write_ints;
		int		mm_ints;
	} info = { 0, 0, 0 };

	hDevice		= NULL;
	hReset		= NULL;
	ReadIoStatus	= NULL;
	WriteIoStatus	= NULL;

	devname = dev;
	b0=strt;
	b1=inc;
	b2=end;

	if (!mm  ) mm="on";
	if (!ints) ints="on";
	if (!irq ) irq="12";
	else
	if (strcmp(irq,"10")==0) {}
	else
	if (strcmp(irq,"11")==0) {}
	else
	if (strcmp(irq,"12")==0) {}
	else
	if (strcmp(irq,"15")==0) {}
	else irq="12";

	printf("devname(%s)\t",devname); 
	printf("(%d)\t",brd);
	printf("(%s)\t",comm);
	printf("bo(%ld)\t",b0);
	printf("b1(%ld)\t",b1);
	printf("b2(%ld)\n",b2);	
	printf("MasterMode %s\t",   mm);
	printf("Interrupts %s\t", ints);
	printf("IRQ set to %s\t",  irq);

	Board = brd;
	UseComportA	= (tolower(comm [0]) == 'a');
	BlockSize = 250;
	strcpy(Comport, "Comport");		/* First Part of the string is Comport*/
	strcat(Comport, comm);			/* append user supplied code - A or B, ...*/
	printf("comport = (%s)\n",Comport);

	i = 0;
	if ((  mm) && (strcmp(  mm, "off") == 0)) switches[i++] = HE_Switch_NoPCIMasterMode;
	if ((ints) && (strcmp(ints, "off") == 0)) switches[i++] = HE_Switch_NoInterrupts;
	if (irq)
	{
		if (strcmp(irq, "10") == 0) switches[i++] = HE_Switch_IRQ10;
		else
		if (strcmp(irq, "11") == 0) switches[i++] = HE_Switch_IRQ11;
		else
		if (strcmp(irq, "12") == 0) switches[i++] = HE_Switch_IRQ12;
		else
		if (strcmp(irq, "15") == 0) switches[i++] = HE_Switch_IRQ15;
	}
	switches[i] = HE_Switch_Last;
    
#else
int main(int argc, char *argv[])
{
	int i;
	HE_WORD		 Board;
	HE_DWORD	 config;
	HE_BOOLEAN	 ConfigSupported = FALSE;
	char		*outfile, *devname;
	char		 Comport[20];
	int			 nul = 0;
	int			 heron_id = 1;
	struct
	{
		int		read_ints;
		int		write_ints;
		int		mm_ints;
	} info = { 0, 0, 0 };

/*	In the following statement set ConfigSupported to
  	TRUE	If you want the program to use Config (where the board supports it)
  	FALSE	If you want the program to ignore the Config signal
*/

	if(argc<7)
	{
		printf("Usage: writes device board Comport Start Increment End [BlockSize]\n");
		printf("e.g.   writes hep6a 0 A 1000 1000 10000\n");
		exit(1);
	}

	devname = argv[1];
	b0	= atol(argv[4]);
	b1	= atol(argv[5]);
	b2	= atol(argv[6]);

	if(argc==8)
		BlockSize = atol(argv[7]);
	else
		BlockSize = 250;

	if(b2 <= 0)
	{
		printf("Sorry can't end at 0! - %ld\n", b2);
		exit(1);
	}
	if (b1 <= 0) b1 = b2;
	if (b0 <= 0) b0 = b1;
	
	if(BlockSize >0)
	{
		if((b2 % BlockSize) != 0)
		{
			printf("end point (%ld) must be a multiple of block size (%ld)\n",
				b2, BlockSize);
			exit(1);
		}
   	
		if((b1 % BlockSize) != 0)
		{
			printf("increment (%ld) must be a multiple of block size (%ld)\n",
				b1, BlockSize);
			exit(1);
		}
   	
		if((b0 % BlockSize) != 0)
		{
			printf("start (%ld) must be a multiple of block size (%ld)\n",
				b1, BlockSize);
			exit(1);
		}
	}
   	
	Board = (HE_WORD)atoi(argv[2]);
	strcpy(Comport, "Comport");
	strcat(Comport, argv[3]);

    UseComportA		= (tolower(argv[3][0]) == 'a');
#endif

	printf("Start at %ld, inc %ld, end at %ld, BlockSize=%ld on %s (%d: %s)\n",
		b0, b1, b2, BlockSize, devname, Board, Comport);

	if (strcmp(devname, "hep6a")==0)
	{
		outfile = "pc6writ.out";
	}
	else
	{
		outfile = "writes.out";
	}

	Status = HeAlloc(&memHandle, b2<<2);	
	if(Status != HE_OK) error(HeGetLastOsError(memHandle));

	Status = HeLock(memHandle, &test);
	if(Status != HE_OK) error(HeGetLastOsError(memHandle));

	for(i=0; i<b2; i++) test[i] = i;

/*	To illustrate how to use the HeOpen1() routine which requires the 
	Comport identification to be passed as a text string rather than as a 
	number we need to convert the Parameter from the command line into
	suitable text:
*/

/*	The following code is an example of reset separated from
	I/O - i.e. reset on A (on an HEPC3) use Comport A or B.

*/
	hDevice		= NULL;
	hReset		= NULL;
	
	if(!UseComportA)
	{
		Status = HeOpen(devname, Board, ComportA, &hReset);
		if(Status != HE_OK) OurError(HeGetLastOsError(hReset));
	}

	Status = HeOpen1(devname, Board, Comport, &hDevice);
	if(Status != HE_OK) OurError(HeGetLastOsError(hDevice));

	printf("Resetting...\n");
	if(UseComportA)
	{
		Status = HeReset(hDevice);
		if(Status != HE_OK) OurError(HeGetLastOsError(hDevice));
	}
	else
	{
		Status = HeReset(hReset);
		if(Status != HE_OK) OurError(HeGetLastOsError(hReset));
	}


	ReadIoStatus	= NULL;
	WriteIoStatus	= NULL;
	
	Status = HeInitIoStatus(hDevice, &ReadIoStatus);
	if(Status != HE_OK) OurError(HeGetLastOsError(hDevice));

	Status = HeInitIoStatus(hDevice, &WriteIoStatus);
	if(Status != HE_OK) OurError(HeGetLastOsError(hDevice));

	/* ******************************************************************** */
	/*   the task of booting an application to the 'C40 and communicating   */
	/*   with the running application starts here. It requires,		        */
	/*   1. resetting the 'C40                                              */
	/*   2. checking config is 0, i.e 'C40 hasn't booted yet - optional     */
	/*   3. booting code from identify.out (a COFF file)					*/
	/*   4. checking the 'C40 has booted, i.e config now = 1 - optional     */
	/*   5. communicating with the process running on the 'C40 with the     */
	/*	    function identify();											*/
	/* ******************************************************************** */
	
	if(ConfigSupported)
	{
		Status = HeConfig(hDevice, &config);
		if(Status != HE_OK) {
			printf("Either Config not supported or some other OurError\n");
			ConfigSupported = FALSE;
		}
    }
    
	if(ConfigSupported)
	{
		printf("CHECKING CONFIG...\n");
		Status = HeConfig(hDevice, &config);
		if(Status != HE_OK) OurError(HeGetLastOsError(hDevice));
		if (config == 1) {
			printf("OurError. Config = 1\n");
			exit(1);
		}
	}

	if (!(fin = fopen(outfile, READ_BIN)))
	{ printf("\n>> can't open file <%s>\n", outfile);  exit(-1); }

	/*-----------------------------------------------------------------------*/
	/* LOAD THE FILE.  CLOSE IT WHEN FINISHED.                               */
	/*-----------------------------------------------------------------------*/
	printf("Booting writes.out...\n");

	if (strcmp(devname, "hep6a")==0)
	{
	}
	else
	{
		/* For HERON boards, we must first identify what HERON to load. */
		HeApiWrite((char *) &heron_id   , 4);
	}

	if (!cload()) { printf("\n>> error loading file\n"); exit(-1); }
	fclose(fin);

	HeApiWrite((char *) &nul        , 4);
	HeApiWrite((char *) &entry_point, 4);

	write_speed();

	printf("Check whether any interrupts were used: ");
	Status = GetIntsStatus(hDevice, (HE_DWORD *) &info.read_ints);
	if (Status != HE_OK) error(HeGetLastOsError(hDevice));
	printf("read %x, write %x, master mode %x.\n", info.read_ints, info.write_ints, info.mm_ints);

	Status = HeUnlock(memHandle);
	if(Status != HE_OK) error(HeGetLastOsError(memHandle));

	Status = HeFree(&memHandle);
	if(Status != HE_OK) error(HeGetLastOsError(memHandle));

	Status = HeClose(&hDevice);
	if(Status != HE_OK) OurError(HeGetLastOsError(hDevice));

	if(!UseComportA)
	{
		Status = HeClose(&hReset);
		if(Status != HE_OK) OurError(HeGetLastOsError(hReset));
	}
}


/****************************************************************************/
/*                                                                          */
/* MEM_WRITE() - Load a buffer of raw data to the target.                   */
/*                                                                          */
/*   THIS FUNCTION MUST BE CUSTOMIZED FOR YOUR APPLICATION !                */
/*                                                                          */
/*   (As supplied, this function simply dumps the data out to the screen.)  */
/*                                                                          */
/****************************************************************************/
int mem_write(buffer, nbytes, addr, page)
   unsigned char *buffer;                   /* POINTER TO DATA BUFFER       */
   int            nbytes;                   /* NUMBER OF 8-BIT BYTES        */
   T_ADDR         addr;                     /* TARGET DESTINATION ADDRESS   */
   int            page;                     /* TARGET DESTINATION PAGE      */
{
   int debug = 0;

   if (verbose)
   {
      int i;
      printf("\n[%0*lx]:%d:", 2 * sizeof(T_ADDR), addr, nbytes);
      for (i = 0; i < nbytes; ++i) printf("%02x ", buffer[i]);
      printf("\n");
   }

   /*-----------------------------------------------------------------------*/
   /* INSERT CUSTOM CODE HERE TO LOAD TARGET MEMORY.                        */
   /*-----------------------------------------------------------------------*/

   if (debug) printf("Buffer 0x%08x, size %04ld, address 0x%08x, page %04d\n", buffer, nbytes, addr, page);

   HeApiWrite  (    (char *) &nbytes, 4);
   HeApiWrite  (    (char *) &addr  , 4);
   HeApiWrite  (    (char *) buffer, nbytes);

   return 1;
}


/****************************************************************************/
/*                                                                          */
/* SET_RELOC_AMOUNT() - Determine relocation amount for each section.       */
/*                                                                          */
/****************************************************************************/
int set_reloc_amount()
{
  int i;

  for (i = 0; i<  n_sections; ++i) reloc_amount[i] = reloc;
  return 1;
}


/****************************************************************************/
/*                                                                          */
/* LOOKUP_SYM() - Stub for symbol lookup routine.                           */
/*                                                                          */
/****************************************************************************/
void lookup_sym(indx, sym, aux)
   long indx;
   SYMENT *sym;
   AUXENT *aux;
{}


/****************************************************************************/
/*                                                                          */
/* LOAD_SYMS() - Stub for symbol load routine.  This routine is only called */
/*               if the global flag 'need_symbols' is TRUE.                 */
/*                                                                          */
/****************************************************************************/
int load_syms(need_reloc)
   int need_reloc;
{
   return 1;
}



/****************************************************************************/
/*                                                                          */
/* MYALLOC() - Application version of 'malloc()'.                           */
/* MRALLOC() - Application version of 'realloc()'.                          */
/*                                                                          */
/****************************************************************************/
void *myalloc(size)
   int size;
{
   char *p = (char *) malloc(size);
   if (p) return p;

   printf("\n>> out of memory\n");
   exit(-1);

   return NULL;
}

void *mralloc(p, size)
   char *p;
   int size;
{
   p = (char *) realloc(p, size);
   if (p) return p;

   printf("\n>> out of memory\n");
   exit(-1);

   return NULL;
}


/****************************************************************************/
/*                                                                          */
/* LOAD_MSG() - Called by loader to write debug messages.                   */
/*                                                                          */
/****************************************************************************/
void load_msg(a,b,c,d,e,f,g)
char *a;
long  b,c,d,e,f,g;
{
   printf(a,b,c,d,e,f,g);
}
